function [ aLambda, aSigma, aFactor ] = Factor_Gibbs_givenK( mY, iK, iSim )
%Gibbs sampler for a Factor model with given K number of factors.
%Based on the work of Geweke & Zhou (1996) and mostly Lopes & West (2004)
%http://apps.olin.wustl.edu/faculty/zhou/GZ-RFS96.pdf
%https://www.jstor.org/stable/pdf/24307179.pdf
%https://www-jstor-org.proxy.uba.uva.nl:2443/stable/pdf/24307179.pdf

%mY = mF*mLambda' + mEps, size(mY) = iTxiP, size(mF) = iTxiK, size(mLambda) = iKxiP, size(mEps) = iTxiP
%var(mEps) = mSigma, size(mSigma) = iPxiP.

%Initialisation-----------------------------------------------------------%
[iT, iP] = size(mY);
fprintf('Processes: %i\n',iP);
fprintf('Factors: %i\n\n',iK );
%Create arrays to store results.
aLambda = zeros(iK, iP, iSim);
aSigma  = zeros(iP, iP, iSim);
aFactor = zeros(iT, iK, iSim);
%-------------------------------------------------------------------------%


%Priors-------------------------------------------------------------------%
%Lambda, lower triangular matrix.
vMu_0       = 0; %taken from Lopes & West (2004)
dC_0        = 1;
dC_0_inv    = 1/dC_0;
mLambda     = zeros(iP,iK); 
for i=1:iP
    for j=1:min(i,iK)
        if j==i
            mLambda(i,j) = normrnd(vMu_0,sqrt(dC_0));
            while mLambda(i,j)<0
                mLambda(i,j) = normrnd(vMu_0,sqrt(dC_0));
            end
        else
            mLambda(i,j) = normrnd(vMu_0,sqrt(dC_0));
        end
    end
end

%Sigma
dNu     = 2.2; %taken from Lopes & West (2004)
dS      = sqrt(0.0455); 
mSigma  = zeros(iP,iP);
for i=1:iP
    mSigma(i,i) = 1/gamrnd( dNu/2 ,2/(dNu*dS^2) );  
end
%-------------------------------------------------------------------------%


%Gibbs--------------------------------------------------------------------%
mF          = mean(mY,2);  %init factor.
mN_F        = mvnrnd(zeros(iK,1), eye(iK), iT*iSim); %pre sample all multivariate normals for speedup=
mN_Lambda   = mvnrnd(zeros(iK,1), eye(iK), iP*iSim);
mIG_Sigma   = 1./gamrnd( (dNu+iT)/2 ,1, [iP, iSim] ); %set scale to one and multiply by scale later on.
for s=1:iSim
    if mod(s,10^4)==0
        fprintf('Simulation %i\n',s)
    end
    
    %Update Lambda
    for i=1:iK
        mVarL_inv   = dC_0_inv*eye(i)+...
                      1/mSigma(i,i)* (mF(:,1:i)'*mF(:,1:i));
        mVarL       = inv(mVarL_inv);
        mMeanL      = mVarL*( dC_0_inv*vMu_0*ones(i,1) ...
                         + 1/mSigma(i,i)*mF(:,1:i)'*mY(:,i)  ); %#ok<MINV>
        mVarL_chol  = chol(mVarL,'lower');
        mLambda(i,:)= mMeanL + mVarL_chol*mN_Lambda((s-1)*iP+i,:)';
        while mLambda(i,:)<0
            mLambda(i,:)= mMeanL + mVarL_chol*mvnrnd(zeros(i,1),eye(i));
        end
    end
    
    mFF         = mF'*mF;
    for i=iK+1:iP
        mVarL_inv   = dC_0_inv*eye(iK)+...
                      1/mSigma(i,i)* mFF;
        mVarL       = inv(mVarL_inv);
        mMeanL      = mVarL*( dC_0_inv*vMu_0*ones(iK,1) ...
                         + 1/mSigma(i,i)*mF'*mY(:,i)  ); %#ok<MINV>
        mVarL_chol  = chol(mVarL, 'lower');
        mLambda(i,:)= mMeanL + mVarL_chol*mN_Lambda((s-1)*iP+i,:)'; 
    end
    aLambda(:,:,s) = mLambda;
    
    
    %Update Factors.
    mLS_inv         = (mLambda'/mSigma);
    mLS_invL        = mLS_inv*mLambda;
    mVarF           = inv(eye(iK) + mLS_invL);
    mMeanF          = mVarF*mLS_inv*mY';  %#ok<MINV>
    mVarF_chol      = chol(mVarF,'upper');
    mF              = mMeanF' + mN_F( (s-1)*iT+1:(s-1)*iT+iT,:)*mVarF_chol'; %speedup
    aFactor(:,:,s)  = mF;
    
    
    %Update Sigma
    for i=1:iP
        vRes = mY(:,i)-mF*mLambda(i,:)';
        dVar = vRes'*vRes;
        dScale      = (dNu*dS^2+dVar)/2;        %note that this is the scale, but as Matlab's gamrnd is specified in scale rather than rate, so IG ~ 1/Gamma(a,1/b) 
        %mSigma(i,i) = 1/gamrnd( (dNu+iT)/2 ,2/(dNu*dS^2+dVar) );           %inverted gamma with shape 1/b.
        mSigma(i,i) = dScale*mIG_Sigma(i,s);           %inverted gamma with shape 1/b.
    end
    aSigma(:,:,s) = mSigma;
end
%-------------------------------------------------------------------------%
end